/*
//
// BEGIN SONGBIRD GPL
//
// This file is part of the Songbird web player.
//
// Copyright(c) 2005-2008 POTI, Inc.
// http://songbirdnest.com
//
// This file may be licensed under the terms of of the
// GNU General Public License Version 2 (the "GPL").
//
// Software distributed under the License is distributed
// on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either
// express or implied. See the GPL for the specific language
// governing rights and limitations.
//
// You should have received a copy of the GPL along with this
// program. If not, go to http://www.gnu.org/licenses/gpl.html
// or write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
//
// END SONGBIRD GPL
//
*/

if (typeof Cc == 'undefined') var Cc = Components.classes;
if (typeof Ci == 'undefined') var Ci = Components.interfaces;
if (typeof Cu == "undefined") var Cu = Components.utils;
if (typeof Cr == 'undefined') var Cr = Components.results;


/* global variables -------------------------------------------------------*/
var zeroConf = null;
var daapLibManager = null;



/* Make sure we have the javascript modules we're going to use */
if (typeof(XPCOMUtils) == "undefined") {
  Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
  if (!XPCOMUtils)
    throw new Error("Import of XPCOMUtils module failed!");
}



var DaapDiscover = {

  /* nsISupports ----------------------------------------------------------*/
  QueryInterface: XPCOMUtils.generateQI([Ci.sbIServiceListener]),

  /* private data ---------------------------------------------------------*/
  server: {},
  cache: {},

  /* initialize -----------------------------------------------------------*/
  init: function ()
  {
	  if (!(zeroConf instanceof Ci.sbIZeroConf)) {
      zeroConf = Cc["@songbirdnest.com/daap/zeroconf;1"]
                  .getService(Ci.sbIZeroConf); 
    }
    if (!(daapLibManager instanceof Ci.sbIDaapLibraryManager)) {
      daapLibManager = Cc["@songbirdnest.com/daap/library-manager;1"]
                        .getService(Ci.sbIDaapLibraryManager);
    }


    //TODO: make an option for this...
    /* start the service discovery */
    zeroConf.addListener(this, "_daap._tcp.local.");
  },

  /* shutdown -------------------------------------------------------------*/
  shutdown: function ()
  {
    //TODO: check if it was started
    /* stop the service discovery */
    zeroConf.removeListener(this, "_daap._tcp.local.");
  },

  /* sbIServiceListener ---------------------------------------------------*/
  onService: function (aService) 
  {
    if (!aService || !aService.name.length) {
      return;
    }

    /* aService is read-only so copy it */
    //XXXDaniel: don't use var here!!!
    let service = { name:       aService.name,
                    target:     aService.target,
                    ttl:        aService.ttl,
                    address:    aService.address,
                    port:       aService.port,
                    txtRecord:  aService.txtRecord  };
    
    if (service.ttl) {
      this.onServiceAdd(service);
    }
    else {
      this.onServiceRemove(service);
    }

    /* make sure the service is erased */
    service = undefined;
  },


  /* private functions ----------------------------------------------------*/

  onServiceAdd: function (aService)
  {
    if (this.server[aService.name]) {
      /* we already have this service, abort */
      return;
    }

    if (!aService.port) {
      /* we need port, abort */
      return;
    }

    if (aService.address.length) {
      /* save the IP address for later use */
      if (aService.target.length) {
        this.cache[aService.target] = aService.address;
      }
    }
    else if (aService.target.length && this.cache[aService.target]) {
      /* no address use the cached one */
      aService.address = this.cache[aService.target];
    }
    else {
      /* no address, abort */
      return;
    }

    /* get or derive the database id */
    let databaseId = null;
    try {
      databaseId = aService.txtRecord.getPropertyAsACString("Database ID");
    }
    catch (e) {
      /* use address:port as ID */
      databaseId = aService.address + "." + aService.port;
    }

    /* get the machine name (aka server name) */
    let machineName = null;
    try {
      machineName = aService.txtRecord.getPropertyAsACString("Machine Name");
    } catch (e) {
      let end = aService.name.indexOf("._daap._tcp.");
      if (end > 0) {
        machineName = aService.name.substring(0, end);
      }
      else {
        machineName = aService.name;
      }
    }
        
    if (machineName) {
      /* fix encoding */
      let converter = Cc["@mozilla.org/intl/scriptableunicodeconverter"]
                        .createInstance(Ci.nsIScriptableUnicodeConverter);
      converter.charset = "UTF-8";
      try {
        machineName = converter.ConvertToUnicode(machineName);
      }
      catch (e) {}
    }
    else {
      /* cannot get a valid machine name, abort */
      return;
    }

    /* register library */
    daapLibManager.registerLibrary(databaseId, machineName, aService.address, aService.port);
    this.server[aService.name] = databaseId;

  },  //onServiceAdd

  onServiceRemove: function (aService)
  {
    if (!this.server[aService.name]){
      /* we don't have this service, do nothing */
      return;
    }

    /* unregister library */
    let databaseId = this.server[aService.name];
    if(databaseId) {
      daapLibManager.unregisterLibrary(databaseId);
    }

    /* remove the service */
    this.server[aService.name] = undefined;

  },  //onServiceRemove

};


